// #gbadev-promo
// /Mic, 2000 | stabmaster_@hotmail.com | come.to/progzone
//
//
// How to compile:
//  zarmasm -CPU ARM7TDMI -Littleend start.asm
//  zarmasm -CPU ARM7TDMI -Littleend gbp_data.asm
//  zarmcc -c -Wall -Otime -ansic -fpu none -Littleend -cpu ARM7TDMI -apcs /narrow/noswst promo.c -o promo.o
//  zarmlink -bin -first start.o start.o promo.o gbp_data.o -map -ro-base 0x08000000 -o promo.bin
//


#include <stdlib.h>
#include <math.h>
#include "gbal.h"

#define camX	120
#define camY	80
#define camZ	-300


extern u8 lbl_pic1;
extern u8 lbl_pic2;
extern u8 lbl_pic3;


float sine[256];
float points[12] = 
{
 0.0f,-55.0f,0.0f, 
 55.0f,0.0f,0.0f, 
 0.0f,55.0f,0.0f, 
 -55.0f,0.0f,0.0f
};
float points2[12];
long points3[8];
long history[8];

u16 col7,col8;
short col10,col11, delta=-32;
u8 blink=0, wich_bg=0;
u8 *bg=&lbl_pic3;
u8 erase=0;



void RotateZ(float *src, float *dest, int rz, int num_points)
{
int i, offs;
float zsa,zca;
float xr,yr,f1;

 zsa = sine[rz&255];
 zca = sine[(rz+90)&255];  

 offs = 0;
 for (i=0; i<num_points; i++)
 {
  xr = src[offs];
  yr = src[offs+1];

  f1 = xr;
  xr = ((zca*xr)-(zsa*yr));
  yr = ((zsa*f1)+(zca*yr));

  dest[offs] = xr;
  dest[offs+1] = yr;
  offs += 3;
 }
}



void Project(float *src,long *dest,int num_points)
{
int i,si,di;
float sx,sy,sz;
  si = 0;
  di = 0;
  for (i=0; i<num_points; i++)
  {
    sx = src[si]; 
    sy = src[si+1];
    sz = src[si+2];
    si += 3;
    sz += (float)(camZ);
    if (sz!=0)
    {
      dest[di] = (long)((FOCAL*sx)/sz)+camX;
      dest[di+1] = (long)((FOCAL*sy)/sz)+camY;
    }else
    {
      dest[di] = camX;
      dest[di+1] = camY;
    }
    di += 2;
  }
}




void draw_box(long x1, long y1, long x2, long y2, u8 col)
{
  __asm
  {
    mov r2,VRAM

    mov r0,y1

    mov r1,#240
    mul r1,r0,r1
    mov r0,x1
    add r0,r0,r1
    add r0,r0,r2

    mov r2,x2
    mov r1,x1
    sub r2,r2,r1
    add r2,r2,#1

    mov r1,y2
    mov r3,y1
    sub r1,r1,r3
    add r1,r1,#1

    mov r3,#240
    sub r3,r3,r2

    mov r4,col
    mov r5,r4,lsl #8
    orr r4,r4,r5
    mov r5,r4,lsl #16
    orr r4,r4,r5

    boxYloop:
      mov r5,r2
      mov r6,r5,lsr #2
      and r5,r5,#3
      boxXloop:
        str r4,[r0],#4
        subs r6,r6,#1
      bne boxXloop
      orrs r5,r5,r5
      beq noTail
      boxTail:
        strb r4,[r0],#1
        subs r5,r5,#1
      bne boxTail
      noTail:
      add r0,r0,r3
      subs r1,r1,#1
    bne boxYloop
  }
}




 
int GBA_main()
{
float f,angle;
u8 theta=0;

  for (f=0; f<256; f+=1)
  {
    angle = (f*3.1415926f/128.0f);
    sine[(int)f] = (float)sin(angle);
  }

  gbalSetMode(_240x160x1x256_);
  
  col7 = 16912;
  col8 = 32767;
  col10 = 960;
  col11 = 960;


  while (1)
  {
    RotateZ(&points[0], &points2[0], theta, 4);
    Project(&points2[0], &points3[0], 4);
    theta++;

    gbalSetColor(BACKGROUND, 7, col7);
    gbalSetColor(BACKGROUND, 8, col8);
    gbalSetColor(BACKGROUND, 10, (col10>>1)&480);
    gbalSetColor(BACKGROUND, 11, col11);

    srccopy8(84,70,72,21,bg);
    notnullcopy8(points3[0]-44, points3[1]-7, 88, 14, &lbl_pic1);
    notnullcopy8(points3[2]-44, points3[3]-7, 88, 14, &lbl_pic1);
    notnullcopy8(points3[4]-44, points3[5]-7, 88, 14, &lbl_pic1);
    notnullcopy8(points3[6]-44, points3[7]-7, 88, 14, &lbl_pic1);

    col10 += delta;
    col11 += delta;
    if ((col10==0) || (col10==960))
    {
      delta = -delta;
      if (!col10)
      {
        if (wich_bg)
        {
          bg = &lbl_pic3;
        }else
        {
          bg = &lbl_pic2;
        }
        wich_bg = 1-wich_bg;
      }
    }

    if (blink)
    {
      col7 = 16912;
      col8 = 32767;
    }else
    {
      col7 = 16784;
      col8 = 32575;
    }
    blink = 1-blink;

    gbalVSync();
    // erase...
    draw_box(points3[0]-44,points3[1]-7,points3[0]+44,points3[1]+7,0);
    draw_box(points3[2]-44,points3[3]-7,points3[2]+44,points3[3]+7,0);
    draw_box(points3[4]-44,points3[5]-7,points3[4]+44,points3[5]+7,0);
    draw_box(points3[6]-44,points3[7]-7,points3[6]+44,points3[7]+7,0);

  }

  return 0;
}

       





